package gov.nasa.jpl.aerie.scheduler.constraints.transformers;

import gov.nasa.jpl.aerie.constraints.model.SimulationResults;
import gov.nasa.jpl.aerie.constraints.time.Window;
import gov.nasa.jpl.aerie.constraints.time.Windows;
import gov.nasa.jpl.aerie.merlin.protocol.types.Duration;
import gov.nasa.jpl.aerie.scheduler.constraints.TimeRangeExpression;
import gov.nasa.jpl.aerie.scheduler.model.Plan;

import java.util.List;

public class TransformerEnvelope implements TimeWindowsTransformer {

  private final List<TimeRangeExpression> insideExprs;


  public TransformerEnvelope(final List<TimeRangeExpression> insideExprs) {
    this.insideExprs = insideExprs;
  }

  @Override
  public Windows transformWindows(final Plan plan, final Windows windowsToTransform, final SimulationResults simulationResults) {

    Windows ret = new Windows();
    if(!windowsToTransform.isEmpty()) {
      Duration min = windowsToTransform.maxTimePoint().get(), max = windowsToTransform.minTimePoint().get();
      boolean atLeastOne = false;
      for (var insideExpr : insideExprs) {

        var rangeExpr = insideExpr.computeRange(simulationResults, plan, windowsToTransform);
        if (!rangeExpr.isEmpty()) {
          atLeastOne = true;
          min = Duration.min(min, rangeExpr.minTimePoint().get());
          max = Duration.max(max, rangeExpr.maxTimePoint().get());
        }
      }

      if (atLeastOne) {
        //register new transformed window
        ret.add(Window.between(min, max));
      }
    }

    return ret;

  }
}
